@@ -18,6 +18,7 @@
 #ifdef BTREE
 
 #include <stdio.h>
+#include <errno.h>
 
 #if HAVE_SYS_FILE_H
 #  include <sys/file.h> /* for flock() */
@@ -37,9 +38,12 @@
 #include "manconfig.h"
 #include "lib/error.h"
 #include "lib/flock.h"
+#include "lib/hashtable.h"
 #include "mydbm.h"
 #include "db_storage.h"
 
+struct hashtable *loop_check_hash;
+
 /* the Berkeley database libraries do nothing to arbitrate between concurrent 
    database accesses, so we do a simple flock(). If the db is opened in 
    anything but O_RDONLY mode, an exclusive lock is enabled. Otherwise, the 
@@ -90,6 +94,24 @@ DB *btree_flopen(char *filename, int flags, int mode)
 		lock_op = LOCK_SH | LOCK_NB;
 	}
 
+	if (!(flags & O_CREAT)) {
+		/* Berkeley DB thinks that a zero-length file means that
+		 * somebody else is writing it, and sleeps for a few
+		 * seconds to give the writer a chance. All very well, but
+		 * the common case is that the database is just zero-length
+		 * because mandb was interrupted or ran out of disk space or
+		 * something like that - so we check for this case by hand
+		 * and ignore the database if it's zero-length.
+		 */
+		struct stat iszero;
+		if (stat (filename, &iszero) < 0)
+			return NULL;
+		if (iszero.st_size == 0) {
+			errno = EINVAL;
+			return NULL;
+		}
+	}
+
 	if (flags & O_TRUNC) {
 		/* opening the db is destructive, need to lock first */
 		int fd;
@@ -163,13 +185,37 @@ int btree_exists(DB *dbf, datum key)
 static __inline__ datum btree_findkey(DB *dbf, u_int flags)
 {
 	datum key, data;
-	
+
+	if (flags == R_FIRST) {
+		if (loop_check_hash) {
+			hash_free (loop_check_hash);
+			loop_check_hash = NULL;
+		}
+	}
+	if (!loop_check_hash)
+		loop_check_hash = hash_create (&plain_hash_free);
+
 	if (((dbf->seq)(dbf, (DBT *) &key, (DBT *) &data, flags))) {
 		key.dptr = NULL;
 		key.dsize = 0;
 		return key;
 	}
 
+	if (hash_lookup (loop_check_hash, key.dptr, key.dsize)) {
+		/* We've seen this key already, which is broken. Return NULL
+		 * so the caller doesn't go round in circles.
+		 */
+		if (debug)
+			fprintf (stderr, "Corrupt database! Already seen %*s. "
+					 "Attempting to recover ...\n",
+				 key.dsize, key.dptr);
+		key.dptr = NULL;
+		key.dsize = 0;
+		return key;
+	}
+
+	hash_install (loop_check_hash, key.dptr, key.dsize, 0, NULL);
+
 	return copy_datum(key);
 }
 
@@ -258,10 +304,10 @@ int dbstore(struct mandata *in, char *basename)
 				cont = make_content(in);
 				status = replace_if_necessary(in, &old, key, cont);
 				free(cont.dptr);
-				free(old.addr);
+				free_mandata_elements(&old);
 				break;
 			}
-			free(old.addr);
+			free_mandata_elements(&old);
 			status = (dbf->seq)(dbf, (DBT *) &key, (DBT *) &cont, R_NEXT);
 			if (strcmp(key.dptr, basename) != 0) {
 				key.dptr = basename;
@@ -309,7 +355,7 @@ static struct mandata *dblookup(char *page, char *section, int flags)
 		    strncmp(section, info->ext, 
 		    	    flags & EXACT ? strlen(info->ext)
 		    	    : strlen(section)) == 0) ) {
-			free(info->addr);
+			free_mandata_elements(info);
 		} else {
 			null_me = &(info->next);
 			info = (info->next = infoalloc());
