@@ -17,6 +17,7 @@
 #include <stdio.h>
 #include <assert.h>
 #include <errno.h>
+#include <ctype.h>
 
 #if defined(STDC_HEADERS)
 #include <string.h>
@@ -26,7 +27,6 @@
 #elif defined(HAVE_STRINGS_H)
 #include <strings.h>
 #else /* no string(s) header file */
-extern char *strtok();
 extern char *strsep();
 #endif /* STDC_HEADERS */
 
@@ -40,8 +40,6 @@ extern char *strsep();
 extern int errno;
 #endif /* not STDC_HEADERS */
 
-#define NLS_SET	db_lookupSet
-
 #include <libintl.h>
 #define _(String) gettext (String)
 
@@ -55,8 +53,11 @@ extern int errno;
 #if defined(NDBM) || defined (BTREE)
 datum copy_datum (datum dat)
 {
-	if (dat.dptr) 
-		dat.dptr = memcpy (xmalloc (dat.dsize), dat.dptr, dat.dsize);
+	if (dat.dptr) {
+		dat.dptr = memcpy (xmalloc (dat.dsize + 1),
+				   dat.dptr, dat.dsize);
+		dat.dptr[dat.dsize] = '\0';
+	}
 	return dat;
 }
 
@@ -77,7 +78,7 @@ void gripe_corrupt_data(void)
 }
 
 /* deal with situation where we cannot replace a key */
-void gripe_replace_key(char *data)
+void gripe_replace_key(const char *data)
 {
 	error (0, 0,
 	       _( "cannot replace key %s"),
@@ -85,24 +86,52 @@ void gripe_replace_key(char *data)
 	gripe_corrupt_data();
 }
 
+void gripe_bad_multi_key(const char *data)
+{
+	error (0, 0,
+	       _("key %s is missing name component - is this an old db?"),
+	       data);
+}
+
+char *copy_if_set(const char *str)
+{
+	if (STREQ(str, "-"))
+		return NULL;
+	else
+		return xstrdup(str);
+}
+
+const char *dash_if_unset(const char *str)
+{
+	if (str)
+		return str;
+	else
+		return "-";
+}
+
 /* Just print out what would be stored in the db */
-void dbprintf(struct mandata *info)
+void dbprintf(const struct mandata *info)
 {
 	fprintf(stderr,
+		"name:      %s\n"
 		"sec. ext:  %s\n"
 		"section:   %s\n"
 		"comp. ext: %s\n"
 		"id:        %c\n"
 		"st_mtime   %ld\n"
 		"pointer:   %s\n"
+		"filter:    %s\n"
 		"whatis:    %s\n\n",
+		dash_if_unset(info->name),
 		info->ext, info->sec, info->comp,
 		info->id, (long)info->_st_mtime, 
-		info->pointer, info->whatis);
+		info->pointer, info->filter, info->whatis);
 }
 
-/* Form a multi-style key from page and extension info */
-datum make_multi_key(char *page, char *ext)
+/* Form a multi-style key from page and extension info. The page should
+ * *not* be name_to_key()'d - that should only happen to the parent.
+ */
+datum make_multi_key(const char *page, const char *ext)
 {
 	datum key;
 
@@ -115,24 +144,49 @@ datum make_multi_key(char *page, char *ext)
 /* allocate a mandata structure */
 struct mandata *infoalloc(void)
 {
-	return (struct mandata *) xmalloc (sizeof(struct mandata));
+	struct mandata *info = xmalloc (sizeof(struct mandata));
+	memset (info, 0, sizeof *info);
+	return info;
 }
 
-/* go through the linked list of structures, free()ing the `content' and the
-   structs themselves */
+/* Free allocated elements of a mandata structure, but not the structure
+ * itself.
+ */
+void free_mandata_elements(struct mandata *pinfo)
+{
+	if (pinfo->addr)
+		free(pinfo->addr);		/* free the 'content' */
+	if (pinfo->name)
+		free(pinfo->name);		/* free the real name */
+}
+
+/* Go through the linked list of structures, free()ing the 'content' and the
+ * structs themselves.
+ */
 void free_mandata_struct(struct mandata *pinfo)
 {
 	while(pinfo) {
 		struct mandata *next;
 
 		next = pinfo->next;
-		if (pinfo->addr)
-			free(pinfo->addr); 	/* free the `content' */
+		free_mandata_elements(pinfo);
 		free(pinfo);			/* free the structure */
 		pinfo = next;
 	}
 }
- 
+
+/* Get the key that should be used for a given name. The caller is
+ * responsible for freeing the return value.
+ */
+char *name_to_key(const char *name)
+{
+	char *key = xstrdup (name);
+	char *p;
+	for (p = key; *p; ++p)
+		*p = tolower (*p);
+	return key;
+}
+
 /* return char ptr array to the data's fields */
 char **split_data(char *content, char *start[])
 {
@@ -143,7 +197,7 @@ char **split_data(char *content, char *start[])
 		start[count] = strsep(&content, "\t");
 		if (!start[count]) {
 			error (0, 0,
-			       _( "only %d fields in content"),
+			       _("only %d fields in content"),
 			       count);
 			gripe_corrupt_data();
 		}
@@ -151,6 +205,12 @@ char **split_data(char *content, char *start[])
 
 	/* initialise pointer to Nth field (whatis) */
 	start[FIELDS - 1] = content;
+	if (!start[FIELDS - 1]) {
+		error (0, 0,
+		       _( "only %d fields in content"),
+		       FIELDS - 1);
+		gripe_corrupt_data();
+	}
 	
 	return start;
 }
@@ -163,59 +223,70 @@ void split_content(char *cont_ptr, struct mandata *pinfo)
 
 	data = split_data(cont_ptr, start);
 
+	pinfo->name = copy_if_set(*(data++));
 	pinfo->ext = *(data++);
 	pinfo->sec = *(data++);
 	pinfo->_st_mtime = (time_t) atol(*(data++));	/* time_t format */
 	pinfo->id = **(data++);				/* single char id */
 	pinfo->pointer = *(data++);
+	pinfo->filter = *(data++);
 	pinfo->comp = *(data++);
 	pinfo->whatis = *(data);
 
 	pinfo->addr = cont_ptr;
 	pinfo->next = (struct mandata *) NULL;
 }
-	
-/* The compliment of split_content */
+
+/* The complement of split_content */
 datum make_content(struct mandata *in)
 {
 	datum cont;
-	static char dash[] = "-";
+	static const char dash[] = "-";
 
 	if (!in->pointer)
 		in->pointer = dash;
+	if (!in->filter)
+		in->filter = dash;
 	if (!in->comp)
 		in->comp = dash;
 	if (!in->whatis)
 		in->whatis = dash + 1;
 
-	cont.dsize = strlen(in->ext) + 
-	             strlen(in->sec) + 
-	          /* strlen(in->_st_mtime) */ + 11 +
-	          /* strlen(in->id) */ + 1 + 
-	             strlen(in->pointer) + 
-	             strlen(in->comp) + 
-	             strlen(in->whatis) + 7;  
+	cont.dsize = strlen(dash_if_unset(in->name)) +
+		     strlen(in->ext) + 
+		     strlen(in->sec) + 
+		  /* strlen(in->_st_mtime) */ + 11 +
+		  /* strlen(in->id) */ + 1 + 
+		     strlen(in->pointer) + 
+		     strlen(in->filter) + 
+		     strlen(in->comp) + 
+		     strlen(in->whatis) + 8;
 	cont.dptr = (char *) xmalloc (cont.dsize);
 #ifdef ANSI_SPRINTF
-	cont.dsize = 1 + sprintf(cont.dptr, "%s\t%s\t%ld\t%c\t%s\t%s\t%s", 
-	        in->ext,
-	        in->sec,
-	        in->_st_mtime,
-	        in->id, 
-	        in->pointer, 
-	        in->comp, 
-	        in->whatis);
+	cont.dsize = 1 + sprintf(cont.dptr,
+		"%s\t%s\t%s\t%ld\t%c\t%s\t%s\t%s\t%s",
+		dash_if_unset(in->name),
+		in->ext,
+		in->sec,
+		in->_st_mtime,
+		in->id, 
+		in->pointer, 
+		in->filter, 
+		in->comp, 
+		in->whatis);
 
 	assert(strlen(cont.dptr) + 1 == cont.dsize);
 #else /* !ANSI_SPRINTF */
-	sprintf(cont.dptr, "%s\t%s\t%ld\t%c\t%s\t%s\t%s", 
-	        in->ext,
-	        in->sec,
-	        (long)in->_st_mtime,
-	        in->id, 
-	        in->pointer, 
-	        in->comp, 
-	        in->whatis);
+	sprintf(cont.dptr, "%s\t%s\t%s\t%ld\t%c\t%s\t%s\t%s\t%s",
+		dash_if_unset(in->name),
+		in->ext,
+		in->sec,
+		(long)in->_st_mtime,
+		in->id, 
+		in->pointer, 
+		in->filter, 
+		in->comp, 
+		in->whatis);
 
 	cont.dsize = strlen(cont.dptr) + 1;	/* to be sure (st_mtime) */
 #endif /* ANSI_SPRINTF */
@@ -230,21 +301,30 @@ datum make_content(struct mandata *in)
 	return cont;
 }
 
-/* Extract all of the extensions associated with this key */
-int list_extensions(char *data, char *ext[])
+/* Extract all of the names/extensions associated with this key. Each case
+ * variant of a name will be returned separately.
+ */
+int list_extensions(char *data, char *names[], char *ext[])
 {  
 	int count = 0;
 
-	while ( (ext[count] = strsep(&data, "\t")) )
-		count++;
+	while ((names[count] = strsep(&data, "\t")) != NULL) {
+		ext[count] = strsep(&data, "\t");
+		if (ext[count])
+			++count;
+		else
+			break;
+	}
 
 	if (debug)
-		fprintf(stderr, "found %d extensions\n", count);
+		fprintf(stderr, "found %d names/extensions\n", count);
 	return count;
 }
 
-#define	EXACT	1
-#define ALL	0
+/* These should be bitwise-ored together. */
+#define ALL	    0
+#define	EXACT	    1
+#define MATCH_CASE  2
 
 /*
  There are three possibilities on lookup:
@@ -254,69 +334,85 @@ int list_extensions(char *data, char *ext[])
  3) Many items exist. They are all returned, in a multiple structure set.
  */
 #ifndef FAST_BTREE
-static struct mandata *dblookup(char *page, char *section, int flags)
+static struct mandata *dblookup(const char *page, const char *section,
+				int flags)
 {
 	struct mandata *info = NULL;
 	datum key, cont;
 
-	key.dptr = page;
-	key.dsize = strlen(page) + 1;
+	key.dptr = name_to_key(page);
+	key.dsize = strlen(key.dptr) + 1;
 	cont = MYDBM_FETCH(dbf, key);
+	free(key.dptr);
 
 	if (cont.dptr == NULL) {		/* No entries at all */
 		return info;			/* indicate no entries */
 	} else if (*cont.dptr != '\t') {	/* Just one entry */
 		info = infoalloc();
 		split_content(cont.dptr, info);
-		if (section == NULL || 
-		      strncmp(section, info->ext, 
-		              flags & EXACT ? strlen(info->ext) : 
-		                              strlen(section)) == 0) {
-		      	return info;
+		if (!info->name)
+			info->name = xstrdup(page);
+		if (!(flags & MATCH_CASE) || STREQ(info->name, page)) {
+			if (section == NULL || 
+			    STRNEQ(section, info->ext, 
+				   flags & EXACT ? strlen(info->ext) : 
+						   strlen(section))) {
+				return info;
+			}
 		}
 		free_mandata_struct(info);
 		return NULL;
 	} else {				/* multiple entries */
-		char *ext[ENTRIES], **e;
+		char *names[ENTRIES], *ext[ENTRIES];
 		struct mandata *ret = NULL;
+		int refs, i;
 
-		/* Extract all of the extensions associated with this key */
+		/* Extract all of the case-variant-names/extensions
+		 * associated with this key.
+		 */
 
-		(void) list_extensions(cont.dptr + 1, e = ext);
+		refs = list_extensions(cont.dptr + 1, names, ext);
 
 		/* Make the multi keys and look them up */
-		
-		while (*e) {
-			if (section == NULL || 
-			    strncmp(section, *e, 
-			            flags & EXACT ? strlen(*e)
-			                          : strlen(section)) == 0) {
-		                datum multi_cont;
-			                
-			  	key = make_multi_key(page, *e);
-			  	if (debug)
-			  		fprintf(stderr, 
-			  			"multi key lookup (%s)\n", 
-			  		        key.dptr);
-				multi_cont = MYDBM_FETCH(dbf, key);
-				if (multi_cont.dptr == NULL) {
-					error (0, 0,
-					       _(
-						       "bad fetch on multi key %s"),
-					       key.dptr);
-					gripe_corrupt_data();
-				}
-				free(key.dptr);
-					
-				/* allocate info struct, fill it in and
-				   point info to the next in the list */
-				if (!ret)
-					ret = info = infoalloc();
-				else
-					info = info->next = infoalloc();
-				split_content(multi_cont.dptr, info);
+
+		for (i = 0; i < refs; ++i) {
+			datum multi_cont;
+
+			/* Decide whether this part of a multi key is
+			 * suitable.
+			 */
+
+			if ((flags & MATCH_CASE) && !STREQ(names[i], page))
+				continue;
+
+			if (section != NULL &&
+			    !STRNEQ(section, ext[i], 
+			            flags & EXACT ? strlen(ext[i])
+			                          : strlen(section)))
+				continue;
+
+			/* So the key is suitable ... */
+			key = make_multi_key(names[i], ext[i]);
+			if (debug)
+				fprintf(stderr, "multi key lookup (%s)\n", 
+					key.dptr);
+			multi_cont = MYDBM_FETCH(dbf, key);
+			if (multi_cont.dptr == NULL) {
+				error (0, 0, _("bad fetch on multi key %s"),
+				       key.dptr);
+				gripe_corrupt_data();
 			}
-			e++;
+			free(key.dptr);
+				
+			/* allocate info struct, fill it in and
+			   point info to the next in the list */
+			if (!ret)
+				ret = info = infoalloc();
+			else
+				info = info->next = infoalloc();
+			split_content(multi_cont.dptr, info);
+			if (!info->name)
+				info->name = xstrdup(names[i]);
 		}
 		MYDBM_FREE(cont.dptr);
 		return ret;
@@ -324,12 +420,14 @@ static struct mandata *dblookup(char *page, char *section, int flags)
 }
 #endif /* !FAST_BTREE */
 
-struct mandata *dblookup_all(char *page, char *section)
+struct mandata *dblookup_all(const char *page, const char *section,
+			     int match_case)
 {
-	return dblookup(page, section, ALL);
+	return dblookup(page, section, ALL | (match_case ? MATCH_CASE : 0));
 }
 
-struct mandata *dblookup_exact(char *page, char *section)
+struct mandata *dblookup_exact(const char *page, const char *section,
+			       int match_case)
 {
-	return dblookup(page, section, EXACT);
+	return dblookup(page, section, EXACT | (match_case ? MATCH_CASE : 0));
 }
