@@ -40,6 +40,7 @@ extern char *strcpy();
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
 
 #if defined(HAVE_UNISTD_H)
 #  include <unistd.h>
@@ -48,49 +49,47 @@ extern char *strcpy();
 #include "manconfig.h"
 #include "libdb/mydbm.h" /* for full definition of MAN_DB */
 
+#undef MAX
 #define MAX(a,b)	((a)>(b)?a:b)
 
 /* take path, add db name and return */
-char *mkdbname(const char *path)
+char *mkdbname (const char *path)
 {
 	static char *name;
-	size_t len = strlen(path);
+	size_t len = strlen (path);
 
 	name = (char *) xrealloc (name, sizeof MAN_DB + len);
 	(void) strcpy (name, path);
-	(void) strcpy (name+len, MAN_DB);
+	(void) strcpy (name + len, MAN_DB);
 
 	return name;
 }
 
 /*
- * Is file a newer than file b?
+ * Does file a have a different timestamp to file b?
  *
  * case:
  *
  *   a is man_page, b is cat_page
  *
- *   a newer than b         returns    1/3  (ret & 1) == 1
- *   a older than b         returns    0/2  !(ret & 1) == 1
- *   a is zero in length    returns    + 2 (for Wilf. and his stray cats)
- *   b is zero in length    returns    + 4
- *   stat on a fails        returns   -1
- *   stat on b fails        returns   -2
- *   stat on a and b fails  returns   -3
+ *   a and b have different times  returns  1/3  (ret & 1) == 1
+ *   a and b have same times       returns  0/2  !(ret & 1) == 1
+ *   a is zero in length           returns  + 2 (for Wilf. and his stray cats)
+ *   b is zero in length           returns  + 4
+ *   stat on a fails               returns   -1
+ *   stat on b fails               returns   -2
+ *   stat on a and b fails         returns   -3
  */
-int is_newer (char *fa, char *fb)
+int is_changed (const char *fa, const char *fb)
 {
 	struct stat fa_sb;
 	struct stat fb_sb;
 	int fa_stat;
 	int fb_stat;
 	int status = 0;
-#ifndef debug
-	extern int debug;
-#endif
 
 	if (debug)
-		fprintf(stderr, "is_newer: a=%s, b=%s", fa, fb);
+		fprintf (stderr, "is_changed: a=%s, b=%s", fa, fb);
 
 	fa_stat = stat (fa, &fa_sb);
 	if (fa_stat != 0)
@@ -102,32 +101,27 @@ int is_newer (char *fa, char *fb)
 
 	if (status != 0) {
 		if (debug)
-			fprintf(stderr, " (%d)\n", -status);
+			fprintf (stderr, " (%d)\n", -status);
 		return -status;
 	}
 
 	if (fa_sb.st_size == 0)
-		status |= 2;		
+		status |= 2;
 
 	if (fb_sb.st_size == 0)
 		status |= 4;
 
-	/* tar and similar archivers modify the st_mtime of output files.
-	   It is possible to end up with a catfile *seemingly* newer than
-	   a freshly untarred man file.  To get around this, we use 
-	   st_mtime or st_ctime, whichever is greatest (newest). */
-	   
-	status |= (MAX(fa_sb.st_mtime, fa_sb.st_ctime) > fb_sb.st_mtime);
-	
+	status |= (fa_sb.st_mtime != fb_sb.st_mtime);
+
 	if (debug)
-		fprintf(stderr, " (%d)\n", status);
+		fprintf (stderr, " (%d)\n", status);
 	return status;
 }
 
 /*
  * Is path a directory?
  */
-int is_directory (char *path)
+int is_directory (const char *path)
 {
 	struct stat sb;
 	int status;
@@ -143,10 +137,6 @@ int is_directory (char *path)
 /* wrapper for system() */
 int do_system (const char *command)
 {
-#ifndef debug
-	extern int debug;
-#endif
-
   	/*
   	 * If we're debugging, don't really execute the command -- you never
   	 * know what might be in that mangled string :-O.
@@ -159,3 +149,59 @@ int do_system (const char *command)
 		return system (command);
 	}
 }
+
+/* Escape dangerous metacharacters before dumping into a shell command. */
+char *escape_shell (const char *unesc)
+{
+	char *esc, *escp;
+	const char *unescp;
+
+	if (!unesc)
+		return NULL;
+
+	escp = esc = (char *) xmalloc (strlen (unesc) * 2 + 1);
+	for (unescp = unesc; *unescp; unescp++)
+		if ((*unescp >= '0' && *unescp <= '9') ||
+		    (*unescp >= 'A' && *unescp <= 'Z') ||
+		    (*unescp >= 'a' && *unescp <= 'z') ||
+		    strchr (",-./:=@_", *unescp))
+			*escp++ = *unescp;
+		else {
+			*escp++ = '\\';
+			*escp++ = *unescp;
+		}
+	*escp = 0;
+	return esc;
+}
+
+/* Remove a directory and all files in it. Does not recurse beyond that. */
+int remove_directory (const char *directory)
+{
+	DIR *handle = opendir (directory);
+	struct dirent *entry;
+
+	if (!handle)
+		return -1;
+	entry = readdir (handle);
+	while (entry) {
+		struct stat st;
+		char *path = xstrdup (directory);
+		path = strappend (path, "/", entry->d_name, NULL);
+		if (stat (path, &st) == -1) {
+			free (path);
+			return -1;
+		}
+		if (S_ISREG (st.st_mode)) {
+			if (unlink (path) == -1) {
+				free (path);
+				return -1;
+			}
+		}
+		free (path);
+		entry = readdir (handle);
+	}
+
+	if (rmdir (directory) == -1)
+		return -1;
+	return 0;
+}
