@@ -15,6 +15,9 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#include <stdlib.h>
+#endif
 
 #include <libintl.h>
 #define _(String) gettext (String)
@@ -23,10 +26,8 @@
 #include "lib/error.h"
 
 /* print the version message, then exit */
-void ver(void)
+void ver (void)
 {
-	printf(_( "%s, version %s, %s (fpolacco@debian.org)\n"),
-	       program_name, VERSION, DATE);
-	exit(OK);
+	printf (_("%s, version %s, %s\n"), program_name, VERSION, DATE);
+	exit (OK);
 }
-
